mod mcdu;
mod sheet_listing;

use std::{env, path::PathBuf, process};

use gtk::prelude::*;
use mcdu::McduModel;
use relm4::prelude::*;
use sheet_listing::SheetListingModel;

struct AppModel {
    text: String,
    mcdu: Controller<McduModel>,
    sheet_listing: Controller<SheetListingModel>,
}

#[derive(Debug)]
enum AppInput {
    McduInput(char),
    SheetPressed(PathBuf),
}

#[relm4::component]
impl SimpleComponent for AppModel {
    type Input = AppInput;
    type Output = ();
    type Init = PathBuf;

    view! {
        #[root]
        gtk::Window{
            set_default_width: 300,
            set_default_height: 300,
            set_title: Some("Play music!!!"),
            gtk::Box {
                set_orientation: gtk::Orientation::Horizontal,
                gtk::Box {
                    set_orientation: gtk::Orientation::Vertical,
                    set_hexpand: true,
                    gtk::ScrolledWindow {
                        model.sheet_listing.widget(),
                        set_vexpand: true,
                        set_hexpand: true,
                        // set_hscrollbar_policy: PolicyType::Never,
                    },
                },
                gtk::Box {
                    set_orientation: gtk::Orientation::Vertical,
                    set_valign: gtk::Align::Center,
                    model.mcdu.widget(),
                    #[name = "label"]
                    gtk::Label {
                        #[watch]
                        set_label: &model.text,
                        set_margin_all: 5,
                    }
                },
            }
        }
    }

    fn init(
        path: Self::Init,
        window: &Self::Root,
        sender: ComponentSender<Self>,
    ) -> relm4::ComponentParts<Self> {
        relm4_icons::initialize_icons();

        let mcdu = McduModel::builder()
            .launch(())
            .forward(sender.input_sender(), |response| match response {
                mcdu::McduOutput::ButtonPress(c) => AppInput::McduInput(c),
            });

        let sheet_listing =
            SheetListingModel::builder()
                .launch(path)
                .forward(sender.input_sender(), |response| match response {
                    sheet_listing::SheetPressedMessage::SheetPressed(path) => {
                        AppInput::SheetPressed(path)
                    }
                });

        let model = AppModel {
            text: String::from("Text: "),
            mcdu,
            sheet_listing,
        };

        let widgets = view_output!();

        ComponentParts { model, widgets }
    }

    fn update(&mut self, message: Self::Input, _sender: ComponentSender<Self>) {
        match message {
            AppInput::McduInput(c) => self.text.push(c),
            AppInput::SheetPressed(sheet) => opener::open(sheet).unwrap(),
        }
    }
}

fn main() {
    let app = RelmApp::new("de.frajul.sheet-organizer");

    let args: Vec<String> = env::args().collect();

    if args.len() <= 1 {
        eprintln!("Please provide sheet folder path");
        process::exit(1);
    }
    let file_path = PathBuf::from(&args[1]);
    if !file_path.is_dir() {
        eprintln!("Sheet folder path is no dir or does not exist");
        process::exit(1);
    }

    // Pass empty command line args to allow my own parsing
    app.with_args(Vec::new()).run::<AppModel>(file_path);
}
