use relm4::gtk::{glib, prelude::*};
use relm4::{gtk, ComponentParts, ComponentSender, SimpleComponent};

pub struct McduModel;

#[derive(Debug)]
pub enum McduOutput {
    ButtonPress(char),
}

#[relm4::component(pub)]
impl SimpleComponent for McduModel {
    type Init = ();
    type Input = ();
    type Output = McduOutput;

    view! {
        #[root]
        gtk::Grid {
            set_column_spacing: 5,
            set_row_spacing: 5,
        }
    }

    fn init(
        _init: Self::Init,
        root: &Self::Root,
        sender: ComponentSender<Self>,
    ) -> ComponentParts<Self> {
        let alphabet = "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        let mcdu_width = 6;

        for (i, c) in alphabet.char_indices() {
            let button = gtk::Button::default();
            button.set_label(&c.to_string());
            button.connect_clicked(glib::clone!(@strong sender => move |_| {
                sender.output(McduOutput::ButtonPress(c)).unwrap();
            }));

            let column = i % mcdu_width;
            let row = (i - column) / mcdu_width;

            root.attach(&button, column as i32, row as i32, 1, 1);
        }

        let model = McduModel;
        let widgets = view_output!();
        ComponentParts { model, widgets }
    }
}
